import 'dart:math';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class TkBox extends StatelessWidget {
  const TkBox({
    Key key,
    bool show,
    this.color,
    double top,
    double right,
    double bottom,
    double left,
    double vertical,
    double horizontal,
    this.width,
    this.height,
    this.alignment,
    this.child,
    this.onPressed, this.border, this.borderRadius, this.minWidth = 0, this.minHeight = 0, this.decoration,
  })  : assert(vertical == null || top == null, 'Cannot provide both vertical and top.'),
        assert(vertical == null || bottom == null, 'Cannot provide both vertical and bottom.'),
        assert(horizontal == null || right == null, 'Cannot provide both horizontal and right.'),
        assert(horizontal == null || left == null, 'Cannot provide both horizontal and left.'),
        top = top ?? vertical,
        bottom = bottom ?? vertical,
        right = right ?? horizontal,
        left = left ?? horizontal,
        show = show ?? true,
        _random = false,
        super(key: key);

  final Color color;
  final bool show;
  final double top;
  final double right;
  final double bottom;
  final double left;
  final double width;
  final double minWidth;
  final double height;
  final double minHeight;
  final BoxBorder border;
  final BorderRadius borderRadius;
  final AlignmentGeometry alignment;
  final Decoration decoration;
  final Widget child;
  final bool _random;
  final Function onPressed;

  @override
  Widget build(BuildContext context) {
    if (!show) return const SizedBox();
    Widget current = child;

    if (alignment != null) current = Align(alignment: alignment, child: current);

    if (child != null && (top != null || bottom != null || left != null || right != null))
      current = Padding(
          padding: EdgeInsets.only(
              top: top ?? 0.0, bottom: bottom ?? 0.0, right: right ?? 0.0, left: left ?? 0.0),
          child: current);

    var _color = this.color;
    if (_random) {
      var rand = Random();
      int r = (color == null) ? (30 + rand.nextInt(196)) : (color.red + rand.nextInt(256)) ~/ 2;
      int g = (color == null) ? (30 + rand.nextInt(196)) : (color.green + rand.nextInt(256)) ~/ 2;
      int b = (color == null) ? (30 + rand.nextInt(196)) : (color.blue + rand.nextInt(256)) ~/ 2;
      _color = Color.fromARGB(255, r, g, b);
    }

    if (_color != null || border != null || borderRadius != null)
      current = DecoratedBox(
          decoration: BoxDecoration(
            color: color,
            border: border,
            borderRadius: borderRadius,
          ),
          child: current);

    if (decoration != null) {
      current = DecoratedBox(
        decoration: decoration,
        child: current,
      );
    }

    if (minHeight > 0 || minWidth > 0)
      current = ConstrainedBox(
        constraints: BoxConstraints(minWidth: minWidth, minHeight: minHeight),
        child: current,
      );

    if (width != null || height != null)
      current = ConstrainedBox(
        constraints: BoxConstraints.tightFor(width: width, height: height),
        child: current,
      );

    if (onPressed != null)
      current = GestureDetector(
        behavior: HitTestBehavior.opaque,
        onTap: onPressed,
        child: current,
      );

    return current;
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(DiagnosticsProperty<Color>('color', color, defaultValue: null));
    properties.add(DoubleProperty('top', top, defaultValue: null));
    properties.add(DoubleProperty('right', right, defaultValue: null));
    properties.add(DoubleProperty('bottom', bottom, defaultValue: null));
    properties.add(DoubleProperty('left', left, defaultValue: null));
    properties.add(DoubleProperty('width', width, defaultValue: null));
    properties.add(DoubleProperty('height', height, defaultValue: null));
    properties.add(DiagnosticsProperty<AlignmentGeometry>('alignment', alignment,
        showName: false, defaultValue: null));
  }
}